"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseForESLint = void 0;
const visitor_keys_1 = require("../visitor-keys");
const context_1 = require("../context");
const eslint_scope_1 = require("eslint-scope");
const script_1 = require("./script");
const sort_1 = require("./sort");
const template_1 = require("./template");
const analyze_scope_1 = require("./analyze-scope");
const errors_1 = require("../errors");
/**
 * Parse source code
 */
function parseForESLint(code, options) {
    const parserOptions = Object.assign({ ecmaVersion: 2020, sourceType: "module", loc: true, range: true, raw: true, tokens: true, comment: true, eslintVisitorKeys: true, eslintScopeManager: true }, (options || {}));
    parserOptions.sourceType = "module";
    if (parserOptions.ecmaVersion <= 5 || parserOptions.ecmaVersion == null) {
        parserOptions.ecmaVersion = 2015;
    }
    const ctx = new context_1.Context(code, parserOptions);
    const resultScript = script_1.parseScript(ctx.sourceCode.script, parserOptions);
    ctx.readyScopeManager(resultScript.scopeManager);
    const resultTemplate = template_1.parseTemplate(ctx.sourceCode.svelte, ctx, parserOptions);
    analyze_scope_1.analyzeStoreScope(resultScript.scopeManager);
    // Add $$xxx variable
    for (const $$name of ["$$slots", "$$props", "$$restProps"]) {
        const globalScope = resultScript.scopeManager.globalScope;
        const variable = new eslint_scope_1.Variable();
        variable.name = $$name;
        variable.scope = globalScope;
        globalScope.variables.push(variable);
        globalScope.set.set($$name, variable);
        globalScope.through = globalScope.through.filter((reference) => {
            if (reference.identifier.name === $$name) {
                // Links the variable and the reference.
                // And this reference is removed from `Scope#through`.
                reference.resolved = variable;
                variable.references.push(reference);
                return false;
            }
            return true;
        });
    }
    const ast = resultTemplate.ast;
    const statements = [...resultScript.ast.body];
    ast.comments.push(...resultScript.ast.comments);
    sort_1.sort(ast.comments);
    ast.tokens.push(...resultScript.ast.tokens);
    sort_1.sort(ast.tokens);
    ast.sourceType = resultScript.ast.sourceType;
    for (const body of ast.body) {
        if (body.type === "SvelteScriptElement") {
            let statement = statements[0];
            while (statement &&
                body.range[0] <= statement.range[0] &&
                statement.range[1] <= body.range[1]) {
                ;
                statement.parent = body;
                body.body.push(statement);
                statements.shift();
                statement = statements[0];
            }
            if (body.startTag.attributes.every((attr) => attr.type === "SvelteAttribute" &&
                attr.key.name === "context" &&
                attr.value.length === 1 &&
                attr.value[0].type === "SvelteLiteral" &&
                attr.value[0].value === "module")) {
                analyze_scope_1.analyzePropsScope(body, resultScript.scopeManager);
            }
        }
    }
    if (statements.length) {
        throw new errors_1.ParseError("The script is unterminated", statements[0].range[1], ctx);
    }
    resultScript.ast = ast;
    resultScript.services = Object.assign(resultScript.services || {}, {
        isSvelte: true,
        getSvelteHtmlAst() {
            return resultTemplate.svelteAst.html;
        },
    });
    resultScript.visitorKeys = Object.assign({}, visitor_keys_1.KEYS, resultScript.visitorKeys);
    return resultScript;
}
exports.parseForESLint = parseForESLint;
